@@ -1,8 +1,10 @@
+# -*- coding: utf-8 -*-
 # Copyright 2012,2013 Christoph Reiter <reiter.christoph@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of version 2 of the GNU General Public License as
-# published by the Free Software Foundation.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import os
 import sys
@@ -11,19 +13,21 @@ if os.name == "nt" or sys.platform == "darwin":
     from quodlibet.plugins import PluginNotSupportedError
     raise PluginNotSupportedError
 
-import tempfile
-
 from gi.repository import Gtk, GdkPixbuf
+from senf import fsn2uri
 
 import dbus
 import dbus.service
 
+from quodlibet import _
 from quodlibet import app
 from quodlibet.plugins.events import EventPlugin
-from quodlibet.parse import Pattern
-from quodlibet.util.uri import URI
+from quodlibet.pattern import Pattern
+from quodlibet.qltk import Icons
 from quodlibet.util.dbusutils import DBusIntrospectable, DBusProperty
 from quodlibet.util.dbusutils import dbus_unicode_validate as unival
+from quodlibet.util import NamedTemporaryFile
+from quodlibet.compat import iteritems, itervalues
 
 BASE_PATH = "/org/gnome/UPnP/MediaServer2"
 BUS_NAME = "org.gnome.UPnP.MediaServer2.QuodLibet"
@@ -33,9 +37,8 @@ class MediaServer(EventPlugin):
     PLUGIN_ID = "mediaserver"
     PLUGIN_NAME = _("UPnP AV Media Server")
     PLUGIN_DESC = _("Exposes all albums to the Rygel UPnP Media Server "
-                    "through the MediaServer2 D-Bus interface")
-    PLUGIN_ICON = Gtk.STOCK_CONNECT
-    PLUGIN_VERSION = "0.1"
+                    "through the MediaServer2 D-Bus interface.")
+    PLUGIN_ICON = Icons.NETWORK_WORKGROUP
 
     def enabled(self):
         try:
@@ -415,8 +418,8 @@ class SongObject(MediaItem, MediaObject, DBusProperty, DBusIntrospectable,
         dbus.service.FallbackObject.__init__(self, bus, self.PATH)
 
         self.__library = library
-        self.__map = dict((id(v), v) for v in self.__library.itervalues())
-        self.__reverse = dict((v, k) for k, v in self.__map.iteritems())
+        self.__map = dict((id(v), v) for v in itervalues(self.__library))
+        self.__reverse = dict((v, k) for k, v in iteritems(self.__map))
 
         self.__song = DummySongObject(self)
 
@@ -427,7 +430,8 @@ class SongObject(MediaItem, MediaObject, DBusProperty, DBusIntrospectable,
             ("removed", self.__songs_removed),
             ("added", self.__songs_added),
         ]
-        self.__sigs = map(lambda (s, f): self.__library.connect(s, f), signals)
+        self.__sigs = map(
+            lambda x: self.__library.connect(x[0], x[1]), signals)
 
     def __songs_changed(self, lib, songs):
         # We don't know what changed, so get all properties
@@ -435,13 +439,13 @@ class SongObject(MediaItem, MediaObject, DBusProperty, DBusIntrospectable,
 
         for song in songs:
             song_id = str(id(song))
-            # https://code.google.com/p/quodlibet/issues/detail?id=1127
+            # https://github.com/quodlibet/quodlibet/issues/id=1127
             # XXX: Something is emitting wrong changed events..
             # ignore song_ids we don't know for now
             if song_id not in self.__map:
                 continue
             for user in self.__users:
-                # ask the user for the prefix whith which the song is used
+                # ask the user for the prefix with which the song is used
                 prefix = user.get_prefix(song)
                 path = "/" + prefix + "/" + song_id
                 self.emit_properties_changed(MediaItem.IFACE, props, path)
@@ -492,15 +496,16 @@ class AlbumsObject(MediaContainer, MediaObject, DBusPropertyFilter,
         self.__library = library.albums
         self.__library.load()
 
-        self.__map = dict((id(v), v) for v in self.__library.itervalues())
-        self.__reverse = dict((v, k) for k, v in self.__map.iteritems())
+        self.__map = dict((id(v), v) for v in itervalues(self.__library))
+        self.__reverse = dict((v, k) for k, v in iteritems(self.__map))
 
         signals = [
             ("changed", self.__albums_changed),
             ("removed", self.__albums_removed),
             ("added", self.__albums_added),
         ]
-        self.__sigs = map(lambda (s, f): self.__library.connect(s, f), signals)
+        self.__sigs = map(
+            lambda x: self.__library.connect(x[0], x[1]), signals)
 
         self.__dummy = DummyAlbumObject(self)
 
@@ -618,7 +623,7 @@ class Icon(MediaItem, MediaObject, DBusProperty, DBusIntrospectable,
 
         # load into a pixbuf
         theme = Gtk.IconTheme.get_default()
-        pixbuf = theme.load_icon("quodlibet", Icon.SIZE, 0)
+        pixbuf = theme.load_icon(Icons.QUODLIBET, Icon.SIZE, 0)
 
         # make sure the size is right
         pixbuf = pixbuf.scale_simple(Icon.SIZE, Icon.SIZE,
@@ -626,7 +631,7 @@ class Icon(MediaItem, MediaObject, DBusProperty, DBusIntrospectable,
         self.__depth = pixbuf.get_bits_per_sample()
 
         # save and keep reference
-        self.__f = f = tempfile.NamedTemporaryFile()
+        self.__f = f = NamedTemporaryFile()
         pixbuf.savev(f.name, "png", [], [])
 
     def get_property(self, interface, name):
@@ -638,10 +643,10 @@ class Icon(MediaItem, MediaObject, DBusProperty, DBusIntrospectable,
             elif name == "Path":
                 return Icon.PATH
             elif name == "DisplayName":
-                return "I'm an icon \o/"
+                return r"I'm an icon \o/"
         elif interface == MediaItem.IFACE:
             if name == "URLs":
-                return [URI.frompath(self.__f.name)]
+                return [fsn2uri(self.__f.name)]
             elif name == "MIMEType":
                 return "image/png"
             elif name == "Width" or name == "Height":
