@@ -1,37 +1,40 @@
 # -*- coding: utf-8 -*-
 # QLScrobbler: an Audioscrobbler client plugin for Quod Libet.
 # version 0.11
-# (C) 2005-2012 by Joshua Kwan <joshk@triplehelix.org>,
+# (C) 2005-2016 by Joshua Kwan <joshk@triplehelix.org>,
 #                  Joe Wreschnig <piman@sacredchao.net>,
 #                  Franz Pletyz <fpletz@franz-pletz.org>,
 #                  Nicholas J. Michalek <djphazer@gmail.com>,
 #                  Steven Robertson <steven@strobe.cc>
 #                  Nick Boultbee <nick.boultbee@gmail.com>
-# Licensed under GPLv2. See Quod Libet's COPYING for more information.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
-from httplib import HTTPException
-import cPickle as pickle
 import os
 import threading
 import time
-import urllib
-import urllib2
+from hashlib import md5
 
 from gi.repository import Gtk, GLib
 
-try:
-    from hashlib import md5
-except ImportError:
-    from md5 import md5
-
-from quodlibet import config, const, app, util, qltk
+import quodlibet
+from quodlibet import _
+from quodlibet import const, app, util, qltk
 from quodlibet.pattern import Pattern
 from quodlibet.query import Query
 from quodlibet.plugins.events import EventPlugin
-from quodlibet.plugins import PluginConfigMixin
-from quodlibet.qltk.entry import ValidatingEntry, UndoEntry, QueryValidator
+from quodlibet.plugins import PluginConfig
+from quodlibet.qltk.entry import ValidatingEntry, UndoEntry
 from quodlibet.qltk.msg import Message
+from quodlibet.qltk import Icons
 from quodlibet.util.dprint import print_d
+from quodlibet.util.picklehelper import pickle_load, pickle_dump, PickleError
+from quodlibet.compat import urlencode
+from quodlibet.util.urllib import urlopen, UrllibError
+from quodlibet.errorreport import errorhook
 
 
 SERVICES = {
@@ -44,16 +47,40 @@ DEFAULT_TITLEPAT = '<title><version| (<version>)>'
 DEFAULT_ARTISTPAT = '<artist|<artist>|<composer|<composer>|<performer>>>'
 
 
-def config_get(key, default=''):
-    """Returns value for 'key' from config. If key is missing *or empty*,
-    return default."""
-    try:
-        return (config.get("plugins", "scrobbler_%s" % key) or default)
-    except config.Error:
-        return default
+plugin_config = PluginConfig("scrobbler")
+defaults = plugin_config.defaults
+defaults.set("service", DEFAULT_SERVICE)
+defaults.set("titlepat", "")
+defaults.set("artistpat", "")
+defaults.set("url", "")
+defaults.set("username", "")
+defaults.set("password", "")
+defaults.set("exclude", "")
+defaults.set("offline", False)
 
 
-class QLSubmitQueue(PluginConfigMixin):
+def config_get_url():
+    """Gets the URL for the currently configured service.
+    This logic was used often enough to be split out from generic config
+    """
+
+    # TODO: share this between the classes better
+    service = plugin_config.get('service')
+    if service in SERVICES:
+        return SERVICES[service]
+    else:
+        return plugin_config.get('url')
+
+
+def config_get_title_pattern():
+    return plugin_config.get('titlepat') or DEFAULT_TITLEPAT
+
+
+def config_get_artist_pattern():
+    return plugin_config.get('artistpat') or DEFAULT_ARTISTPAT
+
+
+class QLSubmitQueue(object):
     """Manages the submit queue for scrobbles. Works independently of the
     QLScrobbler plugin being enabled; other plugins may use submit() to queue
     songs for scrobbling.
@@ -62,25 +89,13 @@ class QLSubmitQueue(PluginConfigMixin):
     CLIENT = "qlb"
     CLIENT_VERSION = const.VERSION
     PROTOCOL_VERSION = "1.2"
-    DUMP = os.path.join(const.USERDIR, "scrobbler_cache")
-    # This must be the kept the same as `QLScrobbler`
-    CONFIG_SECTION = "scrobbler"
+    DUMP = os.path.join(quodlibet.get_user_dir(), "scrobbler_cache_v2")
 
     # These objects are shared across instances, to allow other plugins to
     # queue scrobbles in future versions of QL
     queue = []
     changed_event = threading.Event()
 
-    def config_get_url(self):
-        """Gets the URL for the currently configured service.
-        This logic was used often enough to be split out from generic config"""
-        # TODO: share this between the classes better
-        service = self.config_get('service', DEFAULT_SERVICE)
-        if service in SERVICES:
-            return SERVICES[service]
-        else:
-            return self.config_get('url')
-
     def set_nowplaying(self, song):
         """Send a Now Playing notification."""
         formatted = self._format_song(song)
@@ -132,35 +147,30 @@ class QLSubmitQueue(PluginConfigMixin):
         self.username, self.password, self.base_url = ('', '', '')
 
         # These need to be set early for _format_song to work
-        self.titlepat = Pattern(
-            self.config_get('titlepat', "") or DEFAULT_TITLEPAT)
-        self.artpat = Pattern(
-            self.config_get('artistpat', "") or DEFAULT_ARTISTPAT)
+        self.titlepat = Pattern(config_get_title_pattern())
+        self.artpat = Pattern(config_get_artist_pattern())
 
         try:
-            disk_queue_file = open(self.DUMP, 'r')
-            disk_queue = pickle.load(disk_queue_file)
-            disk_queue_file.close()
+            with open(self.DUMP, 'rb') as disk_queue_file:
+                disk_queue = pickle_load(disk_queue_file)
             os.unlink(self.DUMP)
             self.queue += disk_queue
-        except Exception:
+        except (EnvironmentError, PickleError):
             pass
 
     @classmethod
     def dump_queue(klass):
         if klass.queue:
             try:
-                disk_queue_file = open(klass.DUMP, 'w')
-                pickle.dump(klass.queue, disk_queue_file)
-                disk_queue_file.close()
-            except IOError:
+                with open(klass.DUMP, 'wb') as disk_queue_file:
+                    pickle_dump(klass.queue, disk_queue_file)
+            except (EnvironmentError, PickleError):
                 pass
-        return 0
 
     def _check_config(self):
-        user = self.config_get('username')
-        passw = md5(self.config_get('password')).hexdigest()
-        url = self.config_get_url()
+        user = plugin_config.get('username')
+        passw = md5(plugin_config.getbytes('password')).hexdigest()
+        url = config_get_url()
         if not user or not passw or not url:
             if self.queue and not self.broken:
                 self.quick_dialog(_("Please visit the Plugins window to set "
@@ -172,11 +182,9 @@ class QLSubmitQueue(PluginConfigMixin):
             self.username, self.password, self.base_url = (user, passw, url)
             self.broken = False
             self.handshake_sent = False
-        self.offline = self.config_get_bool('offline')
-        self.titlepat = Pattern(
-                self.config_get('titlepat', "") or DEFAULT_TITLEPAT)
-        self.artpat = Pattern(
-                self.config_get('artistpat', "") or DEFAULT_ARTISTPAT)
+        self.offline = plugin_config.getboolean('offline')
+        self.titlepat = Pattern(config_get_title_pattern())
+        self.artpat = Pattern(config_get_artist_pattern())
 
     def changed(self):
         """Signal that settings or queue contents were changed."""
@@ -230,15 +238,16 @@ class QLSubmitQueue(PluginConfigMixin):
     def send_handshake(self, show_dialog=False):
         # construct url
         stamp = int(time.time())
-        auth = md5(self.password + str(stamp)).hexdigest()
+        auth = md5(self.password.encode("utf-8") +
+                   str(stamp).encode("utf-8")).hexdigest()
         url = "%s/?hs=true&p=%s&c=%s&v=%s&u=%s&a=%s&t=%d" % (
                     self.base_url, self.PROTOCOL_VERSION, self.CLIENT,
                     self.CLIENT_VERSION, self.username, auth, stamp)
         print_d("Sending handshake to service.")
 
         try:
-            resp = urllib2.urlopen(url)
-        except (IOError, HTTPException):
+            resp = urlopen(url)
+        except UrllibError:
             if show_dialog:
                 self.quick_dialog(
                     _("Could not contact service '%s'.") %
@@ -253,7 +262,7 @@ class QLSubmitQueue(PluginConfigMixin):
             return False
 
         # check response
-        lines = resp.read().rstrip().split("\n")
+        lines = resp.read().decode("utf-8", "ignore").rstrip().split("\n")
         status = lines.pop(0)
         print_d("Handshake status: %s" % status)
 
@@ -276,19 +285,19 @@ class QLSubmitQueue(PluginConfigMixin):
             self.quick_dialog(_("Wrong system time. Submissions may fail "
                             "until it is corrected."), Gtk.MessageType.ERROR)
         else:  # "FAILED"
-            self.quick_dialog(status, Gtk.MessageType.ERROR)
+            self.quick_dialog(util.escape(status), Gtk.MessageType.ERROR)
         self.changed()
         return False
 
     def _check_submit(self, url, data):
-        data_str = urllib.urlencode(data)
+        data_str = urlencode(data).encode("ascii")
         try:
-            resp = urllib2.urlopen(url, data_str)
-        except (IOError, HTTPException):
+            resp = urlopen(url, data_str)
+        except EnvironmentError:
             print_d("Audioscrobbler server not responding, will try later.")
             return False
 
-        resp_save = resp.read()
+        resp_save = resp.read().decode("utf-8", "ignore")
         status = resp_save.rstrip().split("\n")[0]
         print_d("Submission status: %s" % status)
 
@@ -336,19 +345,24 @@ class QLSubmitQueue(PluginConfigMixin):
         GLib.idle_add(self.quick_dialog_helper, dialog_type, msg)
 
 
-class QLScrobbler(EventPlugin, PluginConfigMixin):
+class QLScrobbler(EventPlugin):
     PLUGIN_ID = "QLScrobbler"
     PLUGIN_NAME = _("AudioScrobbler Submission")
     PLUGIN_DESC = _("Audioscrobbler client for Last.fm, Libre.fm and other "
                     "Audioscrobbler services.")
-    PLUGIN_ICON = Gtk.STOCK_CONNECT
-    # Retain original config section
-    CONFIG_SECTION = "scrobbler"
+    PLUGIN_ICON = Icons.NETWORK_WORKGROUP
 
     def __init__(self):
         self.__enabled = False
         self.queue = QLSubmitQueue()
-        queue_thread = threading.Thread(None, self.queue.run)
+
+        def queue_run():
+            try:
+                self.queue.run()
+            except Exception:
+                errorhook()
+
+        queue_thread = threading.Thread(None, queue_run)
         queue_thread.setDaemon(True)
         queue_thread.start()
 
@@ -357,16 +371,7 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
         self.elapsed = 0
         self.nowplaying = None
 
-        self.exclude = self.config_get('exclude')
-
-    def config_get_url(self):
-        """Gets the URL for the currently configured service.
-        This logic was used often enough to be split out from generic config"""
-        service = self.config_get('service', DEFAULT_SERVICE)
-        if service in SERVICES:
-            return SERVICES[service]
-        else:
-            return self.config_get('url')
+        self.exclude = plugin_config.get('exclude')
 
     def plugin_on_song_ended(self, song, stopped):
         if song is None or not self.__enabled:
@@ -435,13 +440,13 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
     def PluginPreferences(self, parent):
         def changed(entry, key):
             if entry.get_property('sensitive'):
-                config.set("plugins", "scrobbler_" + key, entry.get_text())
+                plugin_config.set(key, entry.get_text())
 
         def combo_changed(widget, urlent):
             service = widget.get_active_text()
-            config.set("plugins", "scrobbler_service", service)
+            plugin_config.set("service", service)
             urlent.set_sensitive((service not in SERVICES))
-            urlent.set_text(self.config_get_url())
+            urlent.set_text(config_get_url())
 
         def check_login(*args):
             queue = QLSubmitQueue()
@@ -455,6 +460,7 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
 
         # first frame
         table = Gtk.Table(n_rows=5, n_columns=2)
+        table.props.expand = False
         table.set_col_spacings(6)
         table.set_row_spacings(6)
 
@@ -473,7 +479,7 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
         row = 0
         service_combo = Gtk.ComboBoxText()
         table.attach(service_combo, 1, 2, row, row + 1)
-        cur_service = self.config_get('service')
+        cur_service = plugin_config.get('service')
 
         # Translators: Other service
         other_label = _(u"Other…")
@@ -488,7 +494,7 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
 
         # url
         entry = UndoEntry()
-        entry.set_text(self.config_get('url'))
+        entry.set_text(plugin_config.get('url'))
         entry.connect('changed', changed, 'url')
         service_combo.connect('changed', combo_changed, entry)
         service_combo.emit('changed')
@@ -498,7 +504,7 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
 
         # username
         entry = UndoEntry()
-        entry.set_text(self.config_get('username'))
+        entry.set_text(plugin_config.get('username'))
         entry.connect('changed', changed, 'username')
         table.attach(entry, 1, 2, row, row + 1)
         labels[row].set_mnemonic_widget(entry)
@@ -506,7 +512,7 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
 
         # password
         entry = UndoEntry()
-        entry.set_text(self.config_get('password'))
+        entry.set_text(plugin_config.get('password'))
         entry.set_visibility(False)
         entry.connect('changed', changed, 'password')
         table.attach(entry, 1, 2, row, row + 1)
@@ -514,7 +520,8 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
         row += 1
 
         # verify data
-        button = qltk.Button(_("_Verify account data"), Gtk.STOCK_INFO)
+        button = qltk.Button(_("_Verify account data"),
+                             Icons.DIALOG_INFORMATION)
         button.connect('clicked', check_login)
         table.attach(button, 0, 2, 4, 5)
 
@@ -522,6 +529,7 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
 
         # second frame
         table = Gtk.Table(n_rows=4, n_columns=2)
+        table.props.expand = False
         table.set_col_spacings(6)
         table.set_row_spacings(6)
 
@@ -541,7 +549,7 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
         row = 0
         # artist pattern
         entry = UndoEntry()
-        entry.set_text(self.config_get('artistpat'))
+        entry.set_text(plugin_config.get('artistpat'))
         entry.connect('changed', changed, 'artistpat')
         table.attach(entry, 1, 2, row, row + 1)
         entry.set_tooltip_text(_("The pattern used to format "
@@ -551,7 +559,7 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
 
         # title pattern
         entry = UndoEntry()
-        entry.set_text(self.config_get('titlepat'))
+        entry.set_text(plugin_config.get('titlepat'))
         entry.connect('changed', changed, 'titlepat')
         table.attach(entry, 1, 2, row, row + 1)
         entry.set_tooltip_text(_("The pattern used to format "
@@ -560,8 +568,8 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
         row += 1
 
         # exclude filter
-        entry = ValidatingEntry(QueryValidator)
-        entry.set_text(self.config_get('exclude'))
+        entry = ValidatingEntry(Query.validator)
+        entry.set_text(plugin_config.get('exclude'))
         entry.set_tooltip_text(
                 _("Songs matching this filter will not be submitted."))
         entry.connect('changed', changed, 'exclude')
@@ -570,10 +578,9 @@ class QLScrobbler(EventPlugin, PluginConfigMixin):
         row += 1
 
         # offline mode
-        offline = self.ConfigCheckButton(
+        offline = plugin_config.ConfigCheckButton(
                 _("_Offline mode (don't submit anything)"),
-                'scrobbler_offline')
-        offline.set_active(self.config_get('offline') == "true")
+                'offline', populate=True)
         table.attach(offline, 0, 2, row, row + 1)
 
         box.pack_start(qltk.Frame(_("Submission"), child=table), True, True, 0)
