@@ -1,11 +1,14 @@
 # -*- coding: utf-8 -*-
 # Copyright 2016 0x1777
-#           2016 Nick Boultbee
+#        2016-17 Nick Boultbee
 #           2017 Didier Villevalois
+#           2017 Muges
+#           2017 Eyenseo
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk, Gdk, Gst
 import cairo
@@ -13,8 +16,9 @@ from math import ceil, floor
 
 from quodlibet import _, app
 from quodlibet import print_w
+from quodlibet import util
 from quodlibet.plugins import PluginConfig, IntConfProp, \
-    ConfProp
+    ConfProp, BoolConfProp
 from quodlibet.plugins.events import EventPlugin
 from quodlibet.qltk import Align
 from quodlibet.qltk import Icons
@@ -32,6 +36,7 @@ class WaveformSeekBar(Gtk.Box):
 
         self._player = player
         self._rms_vals = []
+        self._hovering = False
 
         self._elapsed_label = TimeLabel()
         self._remaining_label = TimeLabel()
@@ -46,6 +51,10 @@ class WaveformSeekBar(Gtk.Box):
 
         self._waveform_scale.connect('size-allocate',
                                      self._update_redraw_interval)
+        self._waveform_scale.connect('motion-notify-event',
+                                     self._on_mouse_hover)
+        self._waveform_scale.connect('leave-notify-event',
+                                     self._on_mouse_leave)
 
         self._label_tracker = TimeTracker(player)
         self._label_tracker.connect('tick', self._on_tick_label, player)
@@ -69,20 +78,25 @@ class WaveformSeekBar(Gtk.Box):
         # Close any existing pipeline to avoid leaks
         self._clean_pipeline()
 
+        if not song.is_file:
+            return
+
         command_template = """
-        filesrc name=fs
-        ! decodebin ! audioconvert
+        uridecodebin name=uridec
+        ! audioconvert
         ! level name=audiolevel interval={} post-messages=true
         ! fakesink sync=false"""
         interval = int(song("~#length") * 1E9 / points)
+        if not interval:
+            return
         print_d("Computing data for each %.3f seconds" % (interval / 1E9))
 
         command = command_template.format(interval)
         pipeline = Gst.parse_launch(command)
-        pipeline.get_by_name("fs").set_property("location", song("~filename"))
+        pipeline.get_by_name("uridec").set_property("uri", song("~uri"))
 
         bus = pipeline.get_bus()
-        self._bus_id = bus.connect("message", self._on_bus_message)
+        self._bus_id = bus.connect("message", self._on_bus_message, points)
         bus.add_signal_watch()
 
         pipeline.set_state(Gst.State.PLAYING)
@@ -90,7 +104,8 @@ class WaveformSeekBar(Gtk.Box):
         self._pipeline = pipeline
         self._new_rms_vals = []
 
-    def _on_bus_message(self, bus, message):
+    def _on_bus_message(self, bus, message, points):
+        force_stop = False
         if message.type == Gst.MessageType.ERROR:
             error, debug = message.parse_error()
             print_d("Error received from element {name}: {error}".format(
@@ -100,15 +115,22 @@ class WaveformSeekBar(Gtk.Box):
             structure = message.get_structure()
             if structure.get_name() == "level":
                 rms_db = structure.get_value("rms")
-                # Calculate average of all channels (usually 2)
-                rms_db_avg = sum(rms_db) / len(rms_db)
-                # Normalize dB value to value between 0 and 1
-                rms = pow(10, (rms_db_avg / 20))
-                self._new_rms_vals.append(rms)
+                if rms_db:
+                    # Calculate average of all channels (usually 2)
+                    rms_db_avg = sum(rms_db) / len(rms_db)
+                    # Normalize dB value to value between 0 and 1
+                    rms = pow(10, (rms_db_avg / 20))
+                    self._new_rms_vals.append(rms)
+                    if len(self._new_rms_vals) >= points:
+                        # The audio might be much longer than we anticipated
+                        # and we would get way too many events due to the too
+                        # short interval set.
+                        force_stop = True
             else:
                 print_w("Got unexpected message of type {}"
                         .format(message.type))
-        elif message.type == Gst.MessageType.EOS:
+
+        if message.type == Gst.MessageType.EOS or force_stop:
             self._clean_pipeline()
 
             # Update the waveform with the new data
@@ -155,10 +177,13 @@ class WaveformSeekBar(Gtk.Box):
         self._update(player)
 
     def _on_song_changed(self, library, songs, player):
+        if not player.info:
+            return
         # Check that the currently playing song has changed
-        if player.info and player.info in songs:
+        if player.info in songs:
             # Trigger a re-computation of the waveform
             self._create_waveform(player.info, CONFIG.max_data_points)
+            self._resize_labels(player.info)
             # Only update the label if some tag value changed
             self._update_label(player)
 
@@ -166,6 +191,7 @@ class WaveformSeekBar(Gtk.Box):
         if player.info:
             # Trigger a re-computation of the waveform
             self._create_waveform(player.info, CONFIG.max_data_points)
+            self._resize_labels(player.info)
 
         self._waveform_scale.set_placeholder(True)
         self._update(player, True)
@@ -179,8 +205,12 @@ class WaveformSeekBar(Gtk.Box):
 
     def _update_label(self, player):
         if player.info:
-            # Position in ms, length in seconds
-            position = player.get_position() / 1000.0
+            if self._hovering:
+                # Show the position pointed by the mouse
+                position = self._waveform_scale.get_mouse_position()
+            else:
+                # Show the position of the player (converted in seconds)
+                position = player.get_position() / 1000.0
             length = player.info("~#length")
             remaining = length - position
 
@@ -216,6 +246,49 @@ class WaveformSeekBar(Gtk.Box):
             self._waveform_scale.set_placeholder(True)
             self._waveform_scale.queue_draw()
 
+    def _on_mouse_hover(self, _, event):
+        def clamp(a, x, b):
+            '''Return x if a <= x <= b, else the a or b nearest to x.'''
+            return min(max(x, a), b)
+
+        width = self._waveform_scale.get_allocation().width
+        self._waveform_scale.set_mouse_x_position(clamp(0, event.x, width))
+
+        if self._hovering:
+            (x, y, w, h) = self._waveform_scale.compute_hover_redraw_area()
+            self._waveform_scale.queue_draw_area(x, y, w, h)
+        else:
+            self._waveform_scale.queue_draw()
+
+        self._update_label(self._player)
+        self._hovering = True
+
+    def _on_mouse_leave(self, _, event):
+        self._waveform_scale.set_mouse_x_position(-1)
+        self._waveform_scale.queue_draw()
+
+        self._hovering = False
+        self._update_label(self._player)
+
+    def _resize_labels(self, song):
+        """Resize the labels to make sure there is enough space to display the
+        length of the songs.
+
+        This prevents the waveform from changing size when the position changes
+        from 9:59 to 10:00 for example."""
+        length = util.format_time_display(song("~#length"))
+
+        # Get the width needed to display the length of the song (the text
+        # displayed in the labels will always be shorter than that)
+        layout = self._remaining_label.get_layout()
+        layout.set_text(length, -1)
+        width, height = layout.get_pixel_size()
+
+        # Set it as the minimum width of the labels to prevent them from
+        # changing width
+        self._remaining_label.set_size_request(width, -1)
+        self._elapsed_label.set_size_request(width, -1)
+
 
 class WaveformScale(Gtk.EventBox):
     """The waveform widget."""
@@ -233,6 +306,12 @@ class WaveformScale(Gtk.EventBox):
         self.override_background_color(
             Gtk.StateFlags.NORMAL, Gdk.RGBA(alpha=0))
 
+        self.mouse_position = -1
+        self._last_mouse_position = -1
+        self.add_events(Gdk.EventMask.POINTER_MOTION_MASK)
+
+        self._seeking = False
+
     @property
     def width(self):
         return self.get_allocation().width
@@ -242,6 +321,7 @@ class WaveformScale(Gtk.EventBox):
 
     def reset(self, rms_vals):
         self._rms_vals = rms_vals
+        self._seeking = False
         self.queue_draw()
 
     def compute_redraw_interval(self):
@@ -253,9 +333,19 @@ class WaveformScale(Gtk.EventBox):
 
         # Compute the coarsest time interval for redraws
         length = self._player.info("~#length")
-        return length * 1000 / (width * pixel_ratio - 1)
+        return length * 1000 / max(width * pixel_ratio, 1)
 
     def compute_redraw_area(self):
+        width = self.width
+        last_position_x = self._last_drawn_position * width
+        position_x = self.position * width
+        return self._compute_redraw_area_between(last_position_x, position_x)
+
+    def compute_hover_redraw_area(self):
+        return self._compute_redraw_area_between(self._last_mouse_position,
+                                                 self.mouse_position)
+
+    def _compute_redraw_area_between(self, x1, x2):
         allocation = self.get_allocation()
         width = allocation.width
         height = allocation.height
@@ -265,13 +355,14 @@ class WaveformScale(Gtk.EventBox):
         line_width = 1.0 / pixel_ratio
 
         # Compute the thinnest rectangle to redraw
-        last_position_x = self._last_drawn_position * width
-        position_x = self.position * width
-        x = max(0.0, min(last_position_x, position_x) - line_width * 5)
-        w = min(width, abs(position_x - last_position_x) + line_width * 10)
+        x = max(0.0, min(x1, x2) - line_width * 5)
+        w = min(width, abs(x2 - x1) + line_width * 10)
         return x, 0.0, w, height
 
-    def draw_waveform(self, cr, width, height, elapsed_color, remaining_color):
+    def draw_waveform(self, cr, width, height, elapsed_color, hover_color,
+                      remaining_color, show_current_pos_config):
+        if width == 0 or height == 0:
+            return
         scale_factor = self.get_scale_factor()
         pixel_ratio = float(scale_factor)
         line_width = 1.0 / pixel_ratio
@@ -280,7 +371,7 @@ class WaveformScale(Gtk.EventBox):
 
         value_count = len(self._rms_vals)
         max_value = max(self._rms_vals)
-        ratio_width = value_count / (float(width) * pixel_ratio - 1)
+        ratio_width = value_count / (float(width) * pixel_ratio)
         ratio_height = max_value / half_height
 
         cr.set_line_width(line_width)
@@ -288,42 +379,61 @@ class WaveformScale(Gtk.EventBox):
         cr.set_line_join(cairo.LINE_JOIN_ROUND)
 
         position_width = self.position * width * pixel_ratio
+        mouse_position = self.mouse_position * scale_factor
+
         hw = line_width / 2.0
         # Avoiding object lookups is slightly faster
         data = self._rms_vals
 
-        # There can't be more than one clip rectangle, due to the draws queued
-        # But handle the other case anyway
-        rectangle_list = cr.copy_clip_rectangle_list()
-        (cx, cy, cw, ch) = rectangle_list[0]
-        if len(rectangle_list) > 1:
-            for i in range(1, len(rectangle_list), 1):
-                (ox, oy, ow, oh) = rectangle_list[i]
-                (cx, cy, cw, ch) = \
-                    (min(cx, ox), min(cy, oy), max(cw, ow), max(ch, oh))
+        # Use the clip rectangles to redraw only what is necessary
+        for (cx, cy, cw, ch) in cr.copy_clip_rectangle_list():
+            for x in range(int(floor(cx * pixel_ratio)),
+                           int(ceil((cx + cw) * pixel_ratio)), 1):
 
-        # Use that clip rectangle to redraw only what is necessary
-        for x in range(int(floor(cx * pixel_ratio)),
-                       int(ceil((cx + cw) * pixel_ratio)), 1):
+                if mouse_position >= 0:
+                    if self._seeking:
+                        # The user is seeking (holding mousebutton down)
+                        fg_color = (elapsed_color if x < mouse_position
+                                    else remaining_color)
+                    elif show_current_pos_config:
+                        # Use hover color and elapsed color to display the
+                        # current playing position while hovering
+                        if x < mouse_position:
+                            if x < position_width:
+                                fg_color = elapsed_color
+                            else:
+                                fg_color = hover_color
+                        elif x < position_width:
+                            fg_color = hover_color
+                        else:
+                            fg_color = remaining_color
+                    else:
+                        # The mouse is hovering the seekbar
+                        fg_color = (hover_color if x < mouse_position
+                                    else remaining_color)
+                else:
+                    fg_color = (elapsed_color if x < position_width
+                                else remaining_color)
 
-            fg_color = (elapsed_color if x < position_width
-                        else remaining_color)
-            cr.set_source_rgba(*list(fg_color))
+                cr.set_source_rgba(*list(fg_color))
 
-            # Basic anti-aliasing / oversampling
-            u1 = max(0, int(floor((x - hw) * ratio_width)))
-            u2 = min(int(ceil((x + hw) * ratio_width)), len(data))
-            val = (sum(data[u1:u2]) / (ratio_height * (u2 - u1))
-                   if u1 != u2 else 0.0)
+                # Basic anti-aliasing / oversampling
+                u1 = max(0, int(floor((x - hw) * ratio_width)))
+                u2 = min(int(ceil((x + hw) * ratio_width)), len(data))
+                val = (sum(data[u1:u2]) / (ratio_height * (u2 - u1))
+                       if u1 != u2 else 0.0)
 
-            hx = x / pixel_ratio + hw
-            cr.move_to(hx, half_height - val)
-            cr.line_to(hx, half_height + val)
-            cr.stroke()
+                hx = x / pixel_ratio + hw
+                cr.move_to(hx, half_height - val)
+                cr.line_to(hx, half_height + val)
+                cr.stroke()
 
         self._last_drawn_position = self.position
+        self._last_mouse_position = self.mouse_position
 
     def draw_placeholder(self, cr, width, height, color):
+        if width == 0 or height == 0:
+            return
         scale_factor = self.get_scale_factor()
         pixel_ratio = float(scale_factor)
         line_width = 1.0 / pixel_ratio
@@ -354,17 +464,45 @@ class WaveformScale(Gtk.EventBox):
         context.save()
         context.set_state(Gtk.StateFlags.NORMAL)
         bg_color = context.get_background_color(context.get_state())
-        fg_color = context.get_color(context.get_state())
+        remaining_color = context.get_color(context.get_state())
         context.restore()
 
         elapsed_color = get_fg_highlight_color(self)
 
-        # Check if the user set a different color in the config
+        # Check if the user set a different elapsed color in the config
         elapsed_color_config = CONFIG.elapsed_color
         if elapsed_color_config and Gdk.RGBA().parse(elapsed_color_config):
             elapsed_color = Gdk.RGBA()
             elapsed_color.parse(elapsed_color_config)
 
+        # Check if the user set a different remaining color in the config
+        remaining_color_config = CONFIG.remaining_color
+        if remaining_color_config and Gdk.RGBA().parse(remaining_color_config):
+            remaining_color = Gdk.RGBA()
+            remaining_color.parse(remaining_color_config)
+
+        # Check if the user set a hover color in the config
+        hover_color_config = CONFIG.hover_color
+        if hover_color_config and Gdk.RGBA().parse(hover_color_config):
+            hover_color = Gdk.RGBA()
+            hover_color.parse(hover_color_config)
+        else:
+            # Generate default hover_color by blending elapsed_color and
+            # remaining_color
+            opacity = 0.4
+            r = (opacity * elapsed_color.alpha * elapsed_color.red +
+                 (1 - opacity) * remaining_color.alpha * remaining_color.red)
+            g = (opacity * elapsed_color.alpha * elapsed_color.green +
+                 (1 - opacity) * remaining_color.alpha * remaining_color.green)
+            b = (opacity * elapsed_color.alpha * elapsed_color.blue +
+                 (1 - opacity) * remaining_color.alpha * remaining_color.blue)
+            a = (opacity * elapsed_color.alpha +
+                 (1 - opacity) * remaining_color.alpha)
+            hover_color = Gdk.RGBA(r, g, b, a)
+
+        # Check if the user turned on showing current position
+        show_current_pos_config = CONFIG.show_current_pos
+
         # Paint the background
         cr.set_source_rgba(*list(bg_color))
         cr.paint()
@@ -373,27 +511,50 @@ class WaveformScale(Gtk.EventBox):
         width = allocation.width
         height = allocation.height
 
-        if not self._placeholder and len(self._rms_vals) > 0:
-            self.draw_waveform(cr, width, height, elapsed_color, fg_color)
+        if not self._placeholder and self._rms_vals:
+            self.draw_waveform(cr, width, height, elapsed_color,
+                               hover_color, remaining_color,
+                               show_current_pos_config)
         else:
-            self.draw_placeholder(cr, width, height, fg_color)
+            self.draw_placeholder(cr, width, height, remaining_color)
 
     def do_button_press_event(self, event):
+        # Left mouse button
+        if event.button == 1 and self._player:
+            self._seeking = True
+            self.queue_draw()
+
+    def do_button_release_event(self, event):
         # Left mouse button
         if event.button == 1 and self._player:
             ratio = event.x / self.get_allocation().width
             length = self._player.info("~#length")
             self._player.seek(ratio * length * 1000)
+            self._seeking = False
+            self.queue_draw()
             return True
 
     def set_position(self, position):
         self.position = position
 
+    def set_mouse_x_position(self, mouse_position):
+        """Set the horizontal position of the mouse in pixel"""
+        self.mouse_position = mouse_position
+
+    def get_mouse_position(self):
+        """Return the position of the song pointed by the mouse in seconds"""
+        ratio = self.mouse_position / self.get_allocation().width
+        length = self._player.info("~#length")
+        return ratio * length
+
 
 class Config(object):
     _config = PluginConfig(__name__)
 
     elapsed_color = ConfProp(_config, "elapsed_color", "")
+    hover_color = ConfProp(_config, "hover_color", "")
+    remaining_color = ConfProp(_config, "remaining_color", "")
+    show_current_pos = BoolConfProp(_config, "show_current_pos", False)
     max_data_points = IntConfProp(_config, "max_data_points", 3000)
 
 CONFIG = Config()
@@ -423,7 +584,7 @@ class WaveformSeekBarPlugin(EventPlugin):
         red = Gdk.RGBA()
         red.parse("#ff0000")
 
-        def changed(entry):
+        def validate_color(entry):
             text = entry.get_text()
 
             if not Gdk.RGBA().parse(text):
@@ -433,22 +594,52 @@ class WaveformSeekBarPlugin(EventPlugin):
                 # Reset text color
                 entry.override_color(Gtk.StateFlags.NORMAL, None)
 
-            CONFIG.elapsed_color = text
+        def elapsed_color_changed(entry):
+            validate_color(entry)
 
+            CONFIG.elapsed_color = entry.get_text()
+
+        def hover_color_changed(entry):
+            validate_color(entry)
+
+            CONFIG.hover_color = entry.get_text()
+
+        def remaining_color_changed(entry):
+            validate_color(entry)
+
+            CONFIG.remaining_color = entry.get_text()
+
+        def on_show_pos_toggled(button, *args):
+            CONFIG.show_current_pos = button.get_active()
         vbox = Gtk.VBox(spacing=6)
 
-        def create_color():
+        def create_color(label_text, color, callback):
             hbox = Gtk.HBox(spacing=6)
             hbox.set_border_width(6)
-            label = Gtk.Label(label=_("Override foreground color:"))
+            label = Gtk.Label(label=label_text)
             hbox.pack_start(label, False, True, 0)
             entry = Gtk.Entry()
-            if CONFIG.elapsed_color:
-                entry.set_text(CONFIG.elapsed_color)
-            entry.connect('changed', changed)
+            if color:
+                entry.set_text(color)
+            entry.connect('changed', callback)
             hbox.pack_start(entry, True, True, 0)
             return hbox
 
-        vbox.pack_start(create_color(), True, True, 0)
+        box = create_color(_("Override foreground color:"),
+                           CONFIG.elapsed_color, elapsed_color_changed)
+        vbox.pack_start(box, True, True, 0)
+
+        box = create_color(_("Override hover color:"), CONFIG.hover_color,
+                           hover_color_changed)
+        vbox.pack_start(box, True, True, 0)
+
+        box = create_color(_("Override remaining color:"),
+                           CONFIG.remaining_color, remaining_color_changed)
+        vbox.pack_start(box, True, True, 0)
+
+        show_current_pos = Gtk.CheckButton(label=_("Show current position"))
+        show_current_pos.set_active(CONFIG.show_current_pos)
+        show_current_pos.connect("toggled", on_show_pos_toggled)
+        vbox.pack_start(show_current_pos, True, True, 0)
 
         return vbox
