@@ -2,8 +2,9 @@
 # Copyright 2004-2006 Joe Wreschnig, Michael Urman, Niklas Janlert
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from mutagen.mp3 import MP3
 
@@ -22,17 +23,16 @@ class MP3File(ID3File):
         self["~#length"] = info.length
         self["~#bitrate"] = int(info.bitrate / 1000)
         self["~format"] = u"MP%d" % info.layer
-        try:
-            # mutagen 1.30+
-            encoder, brm = info.encoder_info, info.bitrate_mode
-        except AttributeError:
-            pass
-        else:
-            brm = {1: u"CBR", 2: u"VBR", 3: u"ABR"}.get(brm, u"")
-            encoding = u"\n".join(filter(None, [encoder, brm]))
-            if encoding:
-                self["~encoding"] = encoding
+        self["~#channels"] = info.channels
+
+        encoder, brm = info.encoder_info, info.bitrate_mode
+        brm = {1: u"CBR", 2: u"VBR", 3: u"ABR"}.get(brm, u"")
+        # encoder_settings is 1.37+
+        settings = getattr(info, "encoder_settings", u"")
+        encoding = u"\n".join(filter(None, [encoder, brm, settings]))
+        if encoding:
+            self["~encoding"] = encoding
 
 
-info = MP3File
+loader = MP3File
 types = [MP3File]
