@@ -1,14 +1,19 @@
+# -*- coding: utf-8 -*-
 # Copyright 2007-2008 Joe Wreschnig
 #           2009,2010 Steven Robertson
 #           2009-2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import GObject
 
-from quodlibet.formats._audio import AudioFile
+from quodlibet.formats import AudioFile
+from quodlibet.util import print_d
+from quodlibet import config
+from quodlibet.compat import listfilter
 
 
 class Equalizer(object):
@@ -56,7 +61,6 @@ class BasePlayer(GObject.GObject, Equalizer):
     name = ""
     version_info = ""
 
-    paused = None
     song = None
     info = None
     volume = None
@@ -68,11 +72,10 @@ class BasePlayer(GObject.GObject, Equalizer):
     # Replay Gain profiles are a list of values to be tried in order;
     # Four things can set them: rg menu, browser, play order, and a default.
     replaygain_profiles = [None, None, None, ["none"]]
-    _volume = 1.0
     _paused = True
     _source = None
 
-    _gsignals_ = {
+    __gsignals__ = {
         'song-started':
         (GObject.SignalFlags.RUN_LAST, None, (object,)),
         'song-ended':
@@ -85,10 +88,15 @@ class BasePlayer(GObject.GObject, Equalizer):
         'error': (GObject.SignalFlags.RUN_LAST, None, (object, object)),
     }
 
-    _gproperties_ = {
+    __gproperties__ = {
         'volume': (float, 'player volume', 'the volume of the player',
-                   0.0, 1.0, 1.0, GObject.PARAM_READWRITE)
-        }
+                   0.0, 1.0, 1.0,
+                   GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
+        'seekable': (bool, 'seekable', 'if the stream is seekable', True,
+                     GObject.ParamFlags.READABLE),
+        'mute': (bool, 'mute', 'if the stream is muted', False,
+                 GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
+    }
 
     def __init__(self, *args, **kwargs):
         super(BasePlayer, self).__init__()
@@ -103,15 +111,73 @@ class BasePlayer(GObject.GObject, Equalizer):
 
         self._destroy()
 
-    def do_get_property(self, property):
-        if property.name == 'volume':
-            return self._volume
-        else:
-            raise AttributeError
+    def calc_replaygain_volume(self, volume):
+        """Returns a new float volume for the given volume.
 
-    def _set_volume(self, v):
+        Takes into account the global active replaygain profile list,
+        the user specified replaygain settings and the tags available
+        for that song.
+
+        Args:
+            volume (float): 0.0..1.0
+        Returns:
+            float: adjusted volume, can be outside of 0.0..0.1
+        """
+
+        if self.song and config.getboolean("player", "replaygain"):
+            profiles = listfilter(None, self.replaygain_profiles)[0]
+            fb_gain = config.getfloat("player", "fallback_gain")
+            pa_gain = config.getfloat("player", "pre_amp_gain")
+            scale = self.song.replay_gain(profiles, pa_gain, fb_gain)
+        else:
+            scale = 1
+        return volume * scale
+
+    def _reset_replaygain(self):
+        self.volume = self.volume
+
+    def reset_replaygain(self):
+        """Call in case something affecting the replaygain adjustment has
+        changed to change the output volume
+        """
+
+        self._reset_replaygain()
+
+    @property
+    def has_external_volume(self):
+        """If setting the volume will affect anything outside of QL and
+        if the volume can change without any event in QL.
+        """
+
+        return False
+
+    @property
+    def volume(self):
+        return self.props.volume
+
+    @volume.setter
+    def volume(self, v):
         self.props.volume = min(1.0, max(0.0, v))
-    volume = property(lambda s: s._volume, _set_volume)
+
+    @property
+    def mute(self):
+        return self.props.mute
+
+    @mute.setter
+    def mute(self, v):
+        self.props.mute = v
+
+    @property
+    def paused(self):
+        raise NotImplementedError
+
+    @property
+    def seekable(self):
+        """If the current song can be seeked, in case it's not clear defaults
+        to True. See the "seekable" GObject property for notifications.
+        """
+
+        return self.props.seekable
 
     def _destroy(self):
         """Clean up"""
@@ -146,6 +212,13 @@ class BasePlayer(GObject.GObject, Equalizer):
 
         raise NotImplementedError
 
+    def sync(self, timeout):
+        """Tries to finish any pending operations. Mainly for testing.
+        timeout in seconds.
+        """
+
+        pass
+
     def get_position(self):
         """The current position in milliseconds"""
 
@@ -166,7 +239,27 @@ class BasePlayer(GObject.GObject, Equalizer):
         self.paused = True
         self.seek(0)
 
-    def reset(self):
+    def play(self):
+        """If a song is active then unpause else reset the source and start
+        playing.
+        """
+
+        if self.song is None:
+            self._reset()
+        else:
+            self.paused = False
+
+    def playpause(self):
+        """If a song is active then toogle the pause mode else reset the
+        source and start playing.
+        """
+
+        if self.song is None:
+            self._reset()
+        else:
+            self.paused ^= True
+
+    def _reset(self):
         """Reset the source and start playing if possible"""
 
         self._source.reset()
@@ -190,7 +283,7 @@ class BasePlayer(GObject.GObject, Equalizer):
         If force is True always go back.
         """
 
-        if force or self.get_position() < 1500:
+        if force or self.get_position() < 1500 or not self.seekable:
             self._source.previous()
             self._end(True)
         else:
@@ -198,7 +291,7 @@ class BasePlayer(GObject.GObject, Equalizer):
         if self.song:
             self.paused = False
 
-    def go_to(self, song_or_iter, explicit=False):
+    def go_to(self, song_or_iter, explicit=False, source=None):
         """Activate the song or iter in the playlist if possible and play it.
 
         Explicit if the action comes from the user.
@@ -208,7 +301,7 @@ class BasePlayer(GObject.GObject, Equalizer):
 
         print_d("Going to %r" % getattr(song_or_iter, "key", song_or_iter))
 
-        if self._source.go_to(song_or_iter, explicit):
+        if self._source.go_to(song_or_iter, explicit, source):
             self._end(True)
         else:
             if isinstance(song_or_iter, AudioFile):
@@ -223,6 +316,6 @@ class BasePlayer(GObject.GObject, Equalizer):
         return self.song is not None
 
     def can_play_uri(self, uri):
-        """Whether the player supports playing te given URI scheme"""
+        """Whether the player supports playing the given URI scheme"""
 
         raise NotImplementedError
