@@ -3,14 +3,15 @@
 #           2011-2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import gi
 try:
     gi.require_version("Gst", "1.0")
     gi.require_version("GstPbutils", "1.0")
-except ValueError, e:
+except ValueError as e:
     raise ImportError(e)
 
 from gi.repository import Gst, GLib, GstPbutils
@@ -18,12 +19,17 @@ from gi.repository import Gst, GLib, GstPbutils
 from quodlibet import const
 from quodlibet import config
 from quodlibet import util
+from quodlibet import app
+from quodlibet import _
 
 from quodlibet.util import fver, sanitize_tags, MainRunner, MainRunnerError, \
-    MainRunnerAbortedError, MainRunnerTimeoutError
+    MainRunnerAbortedError, MainRunnerTimeoutError, print_w, print_d, \
+    print_e, print_
 from quodlibet.player import PlayerError
 from quodlibet.player._base import BasePlayer
 from quodlibet.qltk.notif import Task
+from quodlibet.compat import iteritems
+from quodlibet.formats.mod import ModFile
 
 from .util import (parse_gstreamer_taglist, TagListWrapper, iter_to_list,
     GStreamerSink, link_many, bin_debug)
@@ -33,6 +39,9 @@ from .prefs import GstPlayerPreferences
 STATE_CHANGE_TIMEOUT = Gst.SECOND * 4
 
 
+const.MinVersions.GSTREAMER.check(Gst.version())
+
+
 class BufferingWrapper(object):
     """A wrapper for a Gst.Element (usually GstPlayBin) which pauses the
     elmement in case buffering is needed, but hides the fact that it does.
@@ -144,43 +153,197 @@ class BufferingWrapper(object):
         self.bin = None
 
 
+def sink_has_external_state(sink):
+    return sink.get_factory().get_name() == "pulsesink"
+
+
+def sink_state_is_valid(sink):
+    if not sink_has_external_state(sink):
+        return True
+
+    # pulsesink volume is only valid in PAUSED/PLAYING
+    # https://bugzilla.gnome.org/show_bug.cgi?id=748577
+    current_state = sink.get_state(0)[1]
+    return current_state >= Gst.State.PAUSED
+
+
+class Seeker(object):
+    """Manages async seeking and position reporting for a pipeline.
+
+    You have to call destroy() before it gets gc'd
+    """
+
+    def __init__(self, playbin, player):
+        self._player = player
+        self._playbin = playbin
+
+        self._last_position = 0
+        self._seek_requests = []
+        self._active_seeks = []
+        self._seekable = False
+
+        bus = playbin.get_bus()
+        bus.add_signal_watch()
+        self._bus_id = bus.connect('message', self._on_message)
+        player.notify("seekable")
+
+    @property
+    def seekable(self):
+        """If the current stream is seekable"""
+
+        return self._seekable
+
+    def destroy(self):
+        """This needs to be called before it gets GC'ed"""
+
+        del self._seek_requests[:]
+        del self._active_seeks[:]
+
+        if self._bus_id:
+            bus = self._playbin.get_bus()
+            bus.disconnect(self._bus_id)
+            bus.remove_signal_watch()
+            self._bus_id = None
+
+        self._player = None
+        self._playbin = None
+
+    def set_position(self, pos):
+        """Set the position. Async and might not succeed.
+
+        Args:
+            pos (int): position in milliseconds
+        """
+
+        pos = max(0, int(pos))
+
+        # We need at least a paused state to seek, if there is non active
+        # or pending, request one async.
+        res, next_state, pending = self._playbin.get_state(timeout=0)
+        if pending != Gst.State.VOID_PENDING:
+            next_state = pending
+        if next_state < Gst.State.PAUSED:
+            self._playbin.set_state(Gst.State.PAUSED)
+
+        self._set_position(self._player.song, pos)
+
+    def get_position(self):
+        """Get the position
+
+        Returns:
+            int: the position in milliseconds
+        """
+
+        if self._seek_requests:
+            self._last_position = self._seek_requests[-1][1]
+        elif self._active_seeks:
+            self._last_position = self._active_seeks[-1][1]
+        else:
+            # While we are actively seeking return the last wanted position.
+            # query_position() returns 0 while in this state
+            ok, p = self._playbin.query_position(Gst.Format.TIME)
+            if ok:
+                p //= Gst.MSECOND
+                # During stream seeking querying the position fails.
+                # Better return the last valid one instead of 0.
+                self._last_position = p
+
+        return self._last_position
+
+    def reset(self):
+        """In case the underlying stream has changed, call this to
+        abort any pending seeking actions and update the seekable state
+        """
+
+        self._last_position = 0
+        del self._seek_requests[:]
+        del self._active_seeks[:]
+        self._refresh_seekable()
+
+    def _refresh_seekable(self):
+        query = Gst.Query.new_seeking(Gst.Format.TIME)
+        if self._playbin.query(query):
+            seekable = query.parse_seeking()[1]
+        elif self._player.song is None:
+            seekable = False
+        else:
+            seekable = True
+
+        if seekable != self._seekable:
+            self._seekable = seekable
+            self._player.notify("seekable")
+
+    def _on_message(self, bus, message):
+        if message.type == Gst.MessageType.ASYNC_DONE:
+            # we only get one ASYNC_DONE for multiple seeks, so flush all
+
+            if self._active_seeks:
+                song, pos = self._active_seeks[-1]
+                if song is self._player.song:
+                    self._player.emit("seek", song, pos)
+                del self._active_seeks[:]
+        elif message.type == Gst.MessageType.STATE_CHANGED:
+            if message.src is self._playbin.bin:
+                new_state = message.parse_state_changed()[1]
+                if new_state >= Gst.State.PAUSED:
+                    self._refresh_seekable()
+
+                    if self._seek_requests:
+                        song, pos = self._seek_requests[-1]
+                        if song is self._player.song:
+                            self._set_position(song, pos)
+                        del self._seek_requests[:]
+
+    def _set_position(self, song, pos):
+        event = Gst.Event.new_seek(
+            1.0, Gst.Format.TIME, Gst.SeekFlags.FLUSH,
+            Gst.SeekType.SET, pos * Gst.MSECOND, Gst.SeekType.NONE, 0)
+
+        if self._playbin.send_event(event):
+            self._active_seeks.append((song, pos))
+        else:
+            self._seek_requests.append((song, pos))
+
+
 class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
-    __gproperties__ = BasePlayer._gproperties_
-    __gsignals__ = BasePlayer._gsignals_
-
-    _paused = True
-    _in_gapless_transition = False
-    _last_position = 0
-
-    bin = None
-    _vol_element = None
-    _use_eq = False
-    _eq_element = None
-
-    __atf_id = None
-    __bus_id = None
-    __source_setup_id = None
-
-    __info_buffer = None
 
     def PlayerPreferences(self):
         return GstPlayerPreferences(self, const.DEBUG)
 
     def __init__(self, librarian=None):
         GStreamerPluginHandler.__init__(self)
-        super(GStreamerPlayer, self).__init__()
-        self.version_info = "GStreamer: %s" % fver(Gst.version())
+        BasePlayer.__init__(self)
+
         self._librarian = librarian
+
+        self.version_info = "GStreamer: %s" % fver(Gst.version())
         self._pipeline_desc = None
-        self._lib_id = librarian.connect("changed", self.__songs_changed)
-        self._active_seeks = []
+
+        self._volume = 1.0
+        self._paused = True
+        self._mute = False
+
+        self._in_gapless_transition = False
         self._active_error = False
+
+        self.bin = None
+        self._seeker = None
+        self._int_vol_element = None
+        self._ext_vol_element = None
+        self._ext_mute_element = None
+        self._use_eq = False
+        self._eq_element = None
+        self.__info_buffer = None
+
+        self._lib_id = librarian.connect("changed", self.__songs_changed)
+        self.__atf_id = None
+        self.__bus_id = None
         self._runner = MainRunner()
 
     def __songs_changed(self, librarian, songs):
         # replaygain values might have changed, recalc volume
         if self.song and self.song in songs:
-            self.volume = self.volume
+            self._reset_replaygain()
 
     def _destroy(self):
         self._librarian.disconnect(self._lib_id)
@@ -194,6 +357,13 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             name += " (%s)" % self._pipeline_desc
         return name
 
+    @property
+    def has_external_volume(self):
+        ext = self._ext_vol_element
+        if ext is None or not sink_has_external_state(ext):
+            return False
+        return True
+
     def _set_buffer_duration(self, duration):
         """Set the stream buffer duration in msecs"""
 
@@ -252,8 +422,8 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         # playbin2 has started to control the volume through pulseaudio,
         # which means the volume property can change without us noticing.
         # Use our own volume element for now until this works with PA.
-        self._vol_element = Gst.ElementFactory.make('volume', None)
-        pipeline.insert(0, self._vol_element)
+        self._int_vol_element = Gst.ElementFactory.make('volume', None)
+        pipeline.insert(0, self._int_vol_element)
 
         # Get all plugin elements and append audio converters.
         # playbin already includes one at the end
@@ -271,35 +441,61 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             assert element is not None, pipeline
             bufbin.add(element)
 
-        PIPELINE_ERROR = PlayerError(_("Could not create GStreamer pipeline"))
-
         if len(pipeline) > 1:
             if not link_many(pipeline):
                 print_w("Linking the GStreamer pipeline failed")
-                self._error(PIPELINE_ERROR)
+                self._error(
+                    PlayerError(_("Could not create GStreamer pipeline")))
                 return False
 
-        # Test to ensure output pipeline can preroll
-        bufbin.set_state(Gst.State.READY)
-        result, state, pending = bufbin.get_state(timeout=STATE_CHANGE_TIMEOUT)
-        if result == Gst.StateChangeReturn.FAILURE:
-            bufbin.set_state(Gst.State.NULL)
-            print_w("Prerolling the GStreamer pipeline failed")
-            self._error(PIPELINE_ERROR)
-            return False
+        # see if the sink provides a volume property, if yes, use it
+        sink_element = pipeline[-1]
+        if isinstance(sink_element, Gst.Bin):
+            sink_element = iter_to_list(sink_element.iterate_recurse)[-1]
+
+        self._ext_vol_element = None
+        if hasattr(sink_element.props, "volume"):
+            self._ext_vol_element = sink_element
+
+            # In case we use the sink volume directly we can increase buffering
+            # without affecting the volume change delay too much and safe some
+            # CPU time... (2x default for now).
+            if hasattr(sink_element.props, "buffer_time"):
+                sink_element.set_property("buffer-time", 400000)
+
+            def ext_volume_notify(*args):
+                # gets called from a thread
+                GLib.idle_add(self.notify, "volume")
+
+            self._ext_vol_element.connect("notify::volume", ext_volume_notify)
+
+        self._ext_mute_element = None
+        if hasattr(sink_element.props, "mute") and \
+                sink_element.get_factory().get_name() != "directsoundsink":
+            # directsoundsink has a mute property but it doesn't work
+            # https://bugzilla.gnome.org/show_bug.cgi?id=755106
+            self._ext_mute_element = sink_element
+
+            def mute_notify(*args):
+                # gets called from a thread
+                GLib.idle_add(self.notify, "mute")
+
+            self._ext_mute_element.connect("notify::mute", mute_notify)
 
         # Make the sink of the first element the sink of the bin
         gpad = Gst.GhostPad.new('sink', pipeline[0].get_static_pad('sink'))
         bufbin.add_pad(gpad)
 
-        self.bin = Gst.ElementFactory.make('playbin', None)
-        assert self.bin
+        bin_ = Gst.ElementFactory.make('playbin', None)
+        assert bin_
 
-        bus = self.bin.get_bus()
+        self.bin = BufferingWrapper(bin_, self)
+        self._seeker = Seeker(self.bin, self)
+
+        bus = bin_.get_bus()
         bus.add_signal_watch()
         self.__bus_id = bus.connect('message', self.__message, self._librarian)
 
-        self.bin = BufferingWrapper(self.bin, self)
         self.__atf_id = self.bin.connect('about-to-finish',
             self.__about_to_finish)
 
@@ -321,30 +517,13 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         flags &= ~(GST_PLAY_FLAG_VIDEO | GST_PLAY_FLAG_TEXT)
         self.bin.set_property("flags", flags)
 
-        # find the (uri)decodebin after setup and use autoplug-sort
-        # to sort elements like decoders
-        def source_setup(*args):
-            def autoplug_sort(decode, pad, caps, factories):
-                def set_prio(x):
-                    i, f = x
-                    i = {
-                        "mad": -1,
-                        "mpg123audiodec": -2
-                    }.get(f.get_name(), i)
-                    return (i, f)
-                return zip(*sorted(map(set_prio, enumerate(factories))))[1]
-
-            for e in iter_to_list(self.bin.iterate_recurse):
-                try:
-                    e.connect("autoplug-sort", autoplug_sort)
-                except TypeError:
-                    pass
-                else:
-                    break
-        self.__source_setup_id = self.bin.connect("source-setup", source_setup)
+        if not self.has_external_volume:
+            # Restore volume/ReplayGain and mute state
+            self.volume = self._volume
+            self.mute = self._mute
 
         # ReplayGain information gets lost when destroying
-        self.volume = self.volume
+        self._reset_replaygain()
 
         if self.song:
             self.bin.set_property('uri', self.song("~uri"))
@@ -364,24 +543,23 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             self.bin.disconnect(self.__atf_id)
             self.__atf_id = None
 
-        if self.__source_setup_id:
-            self.bin.disconnect(self.__source_setup_id)
-            self.__source_setup_id = None
+        if self._seeker is not None:
+            self._seeker.destroy()
+            self._seeker = None
+            self.notify("seekable")
 
         if self.bin:
             self.bin.set_state(Gst.State.NULL)
             self.bin.get_state(timeout=STATE_CHANGE_TIMEOUT)
-            self.bin.set_property('audio-sink', None)
-            self.bin.set_property('video-sink', None)
             # BufferingWrapper cleanup
             self.bin.destroy()
             self.bin = None
 
         self._in_gapless_transition = False
-        self._last_position = 0
-        self._active_seeks = []
 
-        self._vol_element = None
+        self._ext_vol_element = None
+        self._int_vol_element = None
+        self._ext_mute_element = None
         self._eq_element = None
 
     def _rebuild_pipeline(self):
@@ -411,24 +589,25 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             gerror, debug_info = message.parse_error()
             message = u""
             if gerror:
-                message = gerror.message.decode("utf-8").rstrip(".")
+                message = gerror.message.rstrip(".")
             details = None
             if debug_info:
                 # strip the first line, not user friendly
                 debug_info = "\n".join(debug_info.splitlines()[1:])
                 # can contain paths, so not sure if utf-8 in all cases
-                details = debug_info.decode("utf-8", errors="replace")
+                details = debug_info
             self._error(PlayerError(message, details))
-
+        elif message.type == Gst.MessageType.STATE_CHANGED:
+            # pulsesink doesn't notify a volume change on startup
+            # and the volume is only valid in > paused states.
+            if message.src is self._ext_vol_element:
+                self.notify("volume")
+            if message.src is self._ext_mute_element:
+                self.notify("mute")
         elif message.type == Gst.MessageType.STREAM_START:
             if self._in_gapless_transition:
                 print_d("Stream changed")
                 self._end(False)
-        elif message.type == Gst.MessageType.ASYNC_DONE:
-            if self._active_seeks:
-                song, pos = self._active_seeks.pop(0)
-                if song is self.song:
-                    self.emit("seek", song, pos)
         elif message.type == Gst.MessageType.ELEMENT:
             message_name = message.get_structure().get_name()
 
@@ -467,13 +646,33 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         format_desc = get_description(message)
         title = _(u"No GStreamer element found to handle media format")
         error_details = _(u"Media format: %(format-description)s") % {
-            "format-description": format_desc.decode("utf-8")}
+            "format-description": format_desc}
 
         def install_done_cb(plugins_return, *args):
             print_d("Gstreamer plugin install return: %r" % plugins_return)
             Gst.update_registry()
 
         context = GstPbutils.InstallPluginsContext.new()
+
+        # new in 1.6
+        if hasattr(context, "set_desktop_id"):
+            from gi.repository import Gtk
+            context.set_desktop_id(app.id)
+
+        # new in 1.6
+        if hasattr(context, "set_startup_notification_id"):
+            current_time = Gtk.get_current_event_time()
+            context.set_startup_notification_id("_TIME%d" % current_time)
+
+        gdk_window = app.window.get_window()
+        if gdk_window:
+            try:
+                xid = gdk_window.get_xid()
+            except AttributeError:  # non X11
+                pass
+            else:
+                context.set_xid(xid)
+
         res = GstPbutils.install_plugins_async(
             [details], context, install_done_cb, None)
         print_d("Gstreamer plugin install result: %r" % res)
@@ -483,9 +682,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             self._error(PlayerError(title, error_details))
 
     def __about_to_finish_sync(self):
-        """Returns a tuple (ok, next_song). ok is True if the next song
-        should be set.
-        """
+        """Returns the next song uri to play or None"""
 
         print_d("About to finish (sync)")
 
@@ -496,30 +693,37 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         # https://bugzilla.gnome.org/show_bug.cgi?id=695474
         if self.song.multisong:
             print_d("multisong: ignore about to finish")
-            return (False, None)
+            return
+
+        # mod + gapless deadlocks
+        # https://github.com/quodlibet/quodlibet/issues/2780
+        if isinstance(self.song, ModFile):
+            return
 
         if config.getboolean("player", "gst_disable_gapless"):
             print_d("Gapless disabled")
-            return (False, None)
+            return
 
         # this can trigger twice, see issue 987
         if self._in_gapless_transition:
-            return (False, None)
+            return
         self._in_gapless_transition = True
 
         print_d("Select next song in mainloop..")
         self._source.next_ended()
         print_d("..done.")
 
-        return (True, self._source.current)
+        song = self._source.current
+        if song is not None:
+            return song("~uri")
 
     def __about_to_finish(self, playbin):
         print_d("About to finish (async)")
 
         try:
-            ok, song = self._runner.call(self.__about_to_finish_sync,
-                                         priority=GLib.PRIORITY_HIGH,
-                                         timeout=0.5)
+            uri = self._runner.call(self.__about_to_finish_sync,
+                                    priority=GLib.PRIORITY_HIGH,
+                                    timeout=0.5)
         except MainRunnerTimeoutError as e:
             # Due to some locks being held during this signal we can get
             # into a deadlock when a seek or state change event happens
@@ -528,16 +732,15 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             # in a non-gapless transition.
             print_d("About to finish (async): %s" % e)
             return
-        except MainRunnerAbortedError:
+        except MainRunnerAbortedError as e:
             print_d("About to finish (async): %s" % e)
             return
         except MainRunnerError:
             util.print_exc()
             return
 
-        if ok:
+        if uri is not None:
             print_d("About to finish (async): setting uri")
-            uri = song("~uri") if song is not None else None
             playbin.set_property('uri', uri)
         print_d("About to finish (async): done")
 
@@ -545,17 +748,56 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         super(GStreamerPlayer, self).stop()
         self.__destroy_pipeline()
 
+    def do_get_property(self, property):
+        if property.name == 'volume':
+            if self._ext_vol_element is not None and \
+                    sink_has_external_state(self._ext_vol_element) and \
+                    sink_state_is_valid(self._ext_vol_element):
+                # never read back the volume if we don't have to, e.g.
+                # directsoundsink maps volume to an int which makes UI
+                # sliders jump if we read the value back
+                self._volume = self._ext_vol_element.get_property("volume")
+            return self._volume
+        elif property.name == "mute":
+            if self._ext_mute_element is not None and \
+                    sink_has_external_state(self._ext_mute_element) and \
+                    sink_state_is_valid(self._ext_mute_element):
+                self._mute = self._ext_mute_element.get_property("mute")
+            return self._mute
+        elif property.name == "seekable":
+            if self._seeker is not None:
+                return self._seeker.seekable
+            return False
+        else:
+            raise AttributeError
+
+    def _reset_replaygain(self):
+        if not self.bin:
+            return
+
+        v = 1.0 if self._ext_vol_element is not None else self._volume
+        v = self.calc_replaygain_volume(v)
+        v = min(10.0, max(0.0, v))
+        self._int_vol_element.set_property('volume', v)
+
     def do_set_property(self, property, v):
         if property.name == 'volume':
             self._volume = v
-            if self.song and config.getboolean("player", "replaygain"):
-                profiles = filter(None, self.replaygain_profiles)[0]
-                fb_gain = config.getfloat("player", "fallback_gain")
-                pa_gain = config.getfloat("player", "pre_amp_gain")
-                scale = self.song.replay_gain(profiles, pa_gain, fb_gain)
-                v = min(10.0, max(0.0, v * scale)) # volume supports 0..10
-            if self.bin:
-                self._vol_element.set_property('volume', v)
+            if self._ext_vol_element:
+                v = min(10.0, max(0.0, v))
+                self._ext_vol_element.set_property("volume", v)
+            else:
+                v = self.calc_replaygain_volume(v)
+                if self.bin:
+                    v = min(10.0, max(0.0, v))
+                    self._int_vol_element.set_property('volume', v)
+        elif property.name == 'mute':
+            self._mute = v
+            if self._ext_mute_element is not None:
+                self._ext_mute_element.set_property("mute", v)
+            else:
+                if self.bin:
+                    self._int_vol_element.set_property("mute", v)
         else:
             raise AttributeError
 
@@ -563,20 +805,9 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         """Return the current playback position in milliseconds,
         or 0 if no song is playing."""
 
-        p = self._last_position
-        if self.song and self.bin:
-            # While we are actively seeking return the last wanted position.
-            # query_position() returns 0 while in this state
-            if self._active_seeks:
-                return self._active_seeks[-1][1]
-
-            ok, p = self.bin.query_position(Gst.Format.TIME)
-            if ok:
-                p //= Gst.MSECOND
-                # During stream seeking querying the position fails.
-                # Better return the last valid one instead of 0.
-                self._last_position = p
-        return p
+        if self._seeker:
+            return self._seeker.get_position()
+        return 0
 
     @property
     def paused(self):
@@ -599,20 +830,17 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                     # Something wants us to pause between songs, or when
                     # we've got no song playing (probably StopAfterMenu).
                     self.__destroy_pipeline()
-                elif self.song.is_file:
-                    # fast path
+                elif self.seekable:
                     self.bin.set_state(Gst.State.PAUSED)
                 else:
-                    # seekable streams (seem to) have a duration >= 0
-                    ok, d = self.bin.query_duration(Gst.Format.TIME)
-                    if not ok:
-                        d = -1
-
-                    if d >= 0:
-                        self.bin.set_state(Gst.State.PAUSED)
-                    else:
-                        # destroy so that we rebuffer on resume
+                    q = Gst.Query.new_buffering(Gst.Format.DEFAULT)
+                    if self.bin.query(q):
+                        # destroy so that we rebuffer on resume i.e. we don't
+                        # want to continue unseekable streams from where we
+                        # paused but from where we unpaused.
                         self.__destroy_pipeline()
+                    else:
+                        self.bin.set_state(Gst.State.PAUSED)
         else:
             if self.song and self.__init_pipeline():
                 self.bin.set_state(Gst.State.PLAYING)
@@ -632,39 +860,28 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         self.error = True
         self.paused = True
 
-        print_w(unicode(player_error))
+        print_w(player_error)
         self.emit('error', self.song, player_error)
         self._active_error = False
 
     def seek(self, pos):
         """Seek to a position in the song, in milliseconds."""
+
         # Don't allow seeking during gapless. We can't go back to the old song.
         if not self.song or self._in_gapless_transition:
             return
 
         if self.__init_pipeline():
-            # ensure any pending state changes have completed and we have
-            # at least paused state, so we can seek
-            state = self.bin.get_state(timeout=STATE_CHANGE_TIMEOUT)[1]
-            if state < Gst.State.PAUSED:
-                self.bin.set_state(Gst.State.PAUSED)
-                self.bin.get_state(timeout=STATE_CHANGE_TIMEOUT)
+            self._seeker.set_position(pos)
 
-            pos = max(0, int(pos))
-            gst_time = pos * Gst.MSECOND
-            event = Gst.Event.new_seek(
-                1.0, Gst.Format.TIME, Gst.SeekFlags.FLUSH,
-                Gst.SeekType.SET, gst_time, Gst.SeekType.NONE, 0)
-            if self.bin.send_event(event):
-                # to get a good estimate for when get_position fails
-                self._last_position = pos
-                # For cases where we get the position directly after
-                # a seek and the seek is not done, GStreamer returns
-                # a valid 0 position. To prevent this we try to emit seek only
-                # after it is done. Every flushing seek will trigger
-                # an async_done message on the bus, so we queue the seek
-                # event here and emit in the bus message callback.
-                self._active_seeks.append((self.song, pos))
+    def sync(self, timeout):
+        if self.bin is not None:
+            # XXX: This is flaky, try multiple times
+            for i in range(5):
+                self.bin.get_state(Gst.SECOND * timeout)
+                # we have some logic in the main loop, so iterate there
+                while GLib.MainContext.default().iteration(False):
+                    pass
 
     def _end(self, stopped, next_song=None):
         print_d("End song")
@@ -673,7 +890,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         # set the new volume before the signals to avoid delays
         if self._in_gapless_transition:
             self.song = self._source.current
-            self.volume = self.volume
+            self._reset_replaygain()
 
         # We need to set self.song to None before calling our signal
         # handlers. Otherwise, if they try to end the song they're given
@@ -687,12 +904,10 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
 
         # Then, set up the next song.
         self.song = self.info = current
-        self.emit('song-started', self.song)
 
         print_d("Next song")
         if self.song is not None:
             if not self._in_gapless_transition:
-                self.volume = self.volume
                 # Due to extensive problems with playbin2, we destroy the
                 # entire pipeline and recreate it each time we're not in
                 # a gapless transition.
@@ -708,10 +923,18 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                     self.bin.set_state(Gst.State.PLAYING)
         else:
             self.__destroy_pipeline()
-            self.paused = True
 
         self._in_gapless_transition = False
 
+        if self._seeker is not None:
+            # we could have a gapless transition to a non-seekable -> update
+            self._seeker.reset()
+
+        self.emit('song-started', self.song)
+
+        if self.song is None:
+            self.paused = True
+
     def __tag(self, tags, librarian):
         if self.song and self.song.multisong:
             self._fill_stream(tags, librarian)
@@ -737,12 +960,12 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         tags = TagListWrapper(tags, merge=True)
         tags = parse_gstreamer_taglist(tags)
 
-        for key, value in sanitize_tags(tags, stream=False).iteritems():
+        for key, value in iteritems(sanitize_tags(tags, stream=False)):
             if self.song.get(key) != value:
                 changed = True
                 self.song[key] = value
 
-        for key, value in sanitize_tags(tags, stream=True).iteritems():
+        for key, value in iteritems(sanitize_tags(tags, stream=True)):
             if new_info.get(key) != value:
                 info_changed = True
                 new_info[key] = value
@@ -802,12 +1025,4 @@ def init(librarian):
     if Gst.debug_get_default_threshold() == Gst.DebugLevel.NONE:
         Gst.debug_set_default_threshold(Gst.DebugLevel.ERROR)
 
-    if Gst.Element.make_from_uri(
-        Gst.URIType.SRC,
-        "file:///fake/path/for/gst", ""):
-        return GStreamerPlayer(librarian)
-    else:
-        raise PlayerError(
-            _("Unable to open input files"),
-            _("GStreamer has no element to handle reading files. Check "
-              "your GStreamer installation settings."))
+    return GStreamerPlayer(librarian)
