@@ -1,21 +1,24 @@
 # -*- coding: utf-8 -*-
-# Copyright 2004-2012 Joe Wreschnig, Michael Urman, Iñigo Serna, Nick Boultbee
+# Copyright 2004-2012 Joe Wreschnig, Michael Urman, Iñigo Serna
+#           2011-2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import sys
 
 from gi.repository import Gtk, Pango, Gdk
 
+from quodlibet import C_, _
 from quodlibet import qltk
-
 from quodlibet import config
 from quodlibet import util
 
 from quodlibet.util import massagers
 
+from quodlibet.formats import AudioFileError
 from quodlibet.qltk.completion import LibraryValueCompletion
 from quodlibet.qltk.tagscombobox import TagsComboBox, TagsComboBoxEntry
 from quodlibet.qltk.views import RCMHintedTreeView, TreeViewColumn
@@ -28,10 +31,13 @@ from quodlibet.qltk._editutils import EditingPluginHandler, OverwriteWarning
 from quodlibet.qltk._editutils import WriteFailedError
 from quodlibet.qltk import Icons
 from quodlibet.plugins import PluginManager
-from quodlibet.util import connect_obj, gdecode
+from quodlibet.util import connect_obj
+from quodlibet.util.i18n import numeric_phrase
 from quodlibet.util.tags import USER_TAGS, MACHINE_TAGS, sortkey as tagsortkey
 from quodlibet.util.string.splitters import (split_value, split_title,
     split_people, split_album)
+from quodlibet.compat import iteritems, string_types, text_type, listkeys, \
+    listmap, itervalues
 
 
 class Comment(object):
@@ -55,20 +61,20 @@ class Comment(object):
 
     def _paren(self):
         if self.shared:
-            return ngettext('missing from %d song',
-                            'missing from %d songs',
-                            self.missing) % self.missing
+            return numeric_phrase('missing from %d song',
+                                  'missing from %d songs',
+                                  self.missing)
         elif self.complete:
-            return ngettext('different across %d song',
-                            'different across %d songs',
-                            self.total) % self.total
+            return numeric_phrase('different across %d song',
+                                  'different across %d songs',
+                                  self.total)
         else:
-            d = ngettext('different across %d song',
-                          'different across %d songs',
-                          self.have) % self.have
-            m = ngettext('missing from %d song',
-                          'missing from %d songs',
-                          self.missing) % self.missing
+            d = numeric_phrase('different across %d song',
+                               'different across %d songs',
+                               self.have)
+            m = numeric_phrase('missing from %d song',
+                               'missing from %d songs',
+                               self.missing)
             return ", ".join([d, m])
 
     def is_special(self):
@@ -121,7 +127,7 @@ class AudioFileGroup(dict):
             if real_keys_only:
                 iter_func = song.iterrealitems
             else:
-                iter_func = song.iteritems
+                iter_func = song.items
 
             for comment, val in iter_func():
                 keys[comment] = keys.get(comment, 0) + 1
@@ -146,10 +152,10 @@ class AudioFileGroup(dict):
         self._can_change = can_change
 
         # collect comment representations
-        for tag, count in keys.iteritems():
+        for tag, count in iteritems(keys):
             first_value = first[tag]
-            if not isinstance(first_value, basestring):
-                first_value = unicode(first_value)
+            if not isinstance(first_value, string_types):
+                first_value = text_type(first_value)
             shared = all[tag]
             complete = count == total
             if shared and complete:
@@ -188,13 +194,11 @@ class SplitValues(Gtk.ImageMenuItem):
             label=_("Split into _Multiple Values"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         self.set_sensitive(len(split_value(value, spls)) > 1)
 
     def activated(self, tag, value):
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         return [(tag, v) for v in split_value(value, spls)]
 
 
@@ -225,13 +229,11 @@ class SplitTitle(Gtk.ImageMenuItem):
             label=_("Split _Version out of Title"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         self.set_sensitive(bool(split_title(value, spls)[1]))
 
     def activated(self, tag, value):
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         title, versions = split_title(value, spls)
         return [(tag, title)] + [("version", v) for v in versions]
 
@@ -244,13 +246,11 @@ class SplitPerson(Gtk.ImageMenuItem):
         super(SplitPerson, self).__init__(label=self.title, use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         self.set_sensitive(bool(split_people(value, spls)[1]))
 
     def activated(self, tag, value):
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         artist, others = split_people(value, spls)
         return [(tag, artist)] + [(self.needs[0], o) for o in others]
 
@@ -350,7 +350,7 @@ class AddTagDialog(Dialog):
             return self.__tag.tag
 
     def get_value(self):
-        return gdecode(self.__val.get_text())
+        return self.__val.get_text()
 
     def __validate(self, editable, add, invalid, box):
         tag = self.get_tag()
@@ -622,7 +622,8 @@ class EditTags(Gtk.VBox):
                 else:
                     b.connect('activate', self.__menu_activate, view)
 
-                    if (not min(map(self.__songinfo.can_change, b.needs) + [1])
+                    if (not min(listmap(self.__songinfo.can_change, b.needs) +
+                                [1])
                             or comment.is_special()):
                         b.set_sensitive(False)
 
@@ -653,7 +654,7 @@ class EditTags(Gtk.VBox):
         remove.set_sensitive(bool(rows))
 
     def __add_new_tag(self, model, tag, value):
-        assert isinstance(value, unicode)
+        assert isinstance(value, text_type)
         iters = [i for (i, v) in model.iterrows() if v.tag == tag]
         if iters and not self.__songinfo.can_multiple_values(tag):
             title = _("Unable to add tag")
@@ -682,9 +683,9 @@ class EditTags(Gtk.VBox):
                 break
             tag = add.get_tag()
             value = add.get_value()
-            assert isinstance(value, unicode)
+            assert isinstance(value, text_type)
             value = massagers.validate(tag, value)
-            assert isinstance(value, unicode)
+            assert isinstance(value, text_type)
             if not self.__songinfo.can_change(tag):
                 title = _("Invalid tag")
                 msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
@@ -716,7 +717,7 @@ class EditTags(Gtk.VBox):
         added = {}
         renamed = {}
 
-        for entry in model.itervalues():
+        for entry in itervalues(model):
             if entry.edited and not (entry.deleted or entry.renamed):
                 if entry.origvalue is not None:
                     l = updated.setdefault(entry.tag, [])
@@ -749,7 +750,7 @@ class EditTags(Gtk.VBox):
                     break
 
             changed = False
-            for key, values in updated.iteritems():
+            for key, values in iteritems(updated):
                 for (new_value, old_value) in values:
                     if song.can_change(key):
                         if old_value is None:
@@ -758,13 +759,13 @@ class EditTags(Gtk.VBox):
                             song.change(key, old_value.text, new_value.text)
                         changed = True
 
-            for key, values in added.iteritems():
+            for key, values in iteritems(added):
                 for value in values:
                     if song.can_change(key):
                         song.add(key, value.text)
                         changed = True
 
-            for key, values in deleted.iteritems():
+            for key, values in iteritems(deleted):
                 for value in values:
                     if not value.shared:
                         # In case it isn't shared we don't know the actual
@@ -778,7 +779,7 @@ class EditTags(Gtk.VBox):
                         changed = True
 
             save_rename = []
-            for new_tag, values in renamed.iteritems():
+            for new_tag, values in iteritems(renamed):
                 for old_tag, new_value, old_value in values:
                     if (song.can_change(new_tag) and old_tag in song):
                         if not new_value.is_special():
@@ -799,7 +800,7 @@ class EditTags(Gtk.VBox):
             if changed:
                 try:
                     song.write()
-                except:
+                except AudioFileError:
                     util.print_exc()
                     WriteFailedError(self, song).run()
                     library.reload(song, changed=was_changed)
@@ -817,8 +818,8 @@ class EditTags(Gtk.VBox):
             b.set_sensitive(not all_done)
 
     def __edit_tag(self, renderer, path, new_value, model):
-        new_value = new_value.decode("utf-8")
-        new_value = ', '.join(new_value.splitlines())
+        #pfps leaving the newline should be OK
+        #        new_value = ', '.join(new_value.splitlines())
         path = Gtk.TreePath.new_from_string(path)
         entry = model[path][0]
         error_dialog = None
@@ -834,8 +835,9 @@ class EditTags(Gtk.VBox):
 
         comment = entry.value
         changed = comment.text != new_value
-        if (changed and ((comment.shared and comment.complete) or new_value)) \
-                or (new_value and comment.shared and not comment.complete):
+        identical = comment.shared and comment.complete
+        if (changed and (identical or new_value)) \
+                or (new_value and not identical):
             # only give an error if we would have applied the value
             if error_dialog is not None:
                 error_dialog.run()
@@ -926,7 +928,7 @@ class EditTags(Gtk.VBox):
             self.__songinfo = AudioFileGroup(songs)
         songinfo = self.__songinfo
 
-        keys = songinfo.keys()
+        keys = listkeys(songinfo)
         default_tags = get_default_tags()
         keys = set(keys + default_tags)
 
