@@ -3,8 +3,9 @@
 #           2011, 2012 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import math
 
@@ -14,7 +15,6 @@ from quodlibet import _
 from quodlibet.qltk import is_accel, add_fake_accel
 from quodlibet.qltk.x import SeparatorMenuItem, MenuItem
 from quodlibet.qltk import Icons
-from quodlibet.util import gdecode
 from quodlibet.compat import string_types
 
 
@@ -96,7 +96,7 @@ class EditableUndo(object):
             menu.prepend(item)
 
     def __all(self):
-        text = gdecode(self.get_chars(0, -1))
+        text = self.get_chars(0, -1)
         pos = self.get_position()
         return [text, pos]
 
@@ -254,7 +254,7 @@ class ValidatingEntryMixin(object):
             self.connect('changed', self.__color, validator)
 
     def __color(self, widget, validator):
-        value = validator(gdecode(self.get_text()))
+        value = validator(self.get_text())
         if value is True:
             color = self.VALID
         elif value is False:
