@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2009 Joe Wreschnig, Michael Urman, Steven Robertson
-#           2011,2013 Nick Boultbee
+#           2011-2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import os
 import random
@@ -12,11 +13,8 @@ import re
 import ctypes
 import ctypes.util
 import sys
-import traceback
 import unicodedata
 import threading
-import subprocess
-import webbrowser
 
 # Windows doesn't have fcntl, just don't lock for now
 try:
@@ -25,35 +23,26 @@ try:
 except ImportError:
     fcntl = None
 
-from quodlibet.compat import reraise as py_reraise, urlparse, PY2, text_type, \
-    iteritems
-from quodlibet.util.path import iscommand
+from senf import fsnative, argv
+
+from quodlibet.compat import reraise as py_reraise, text_type, \
+    iteritems, reduce, number_types, long
 from quodlibet.util.string.titlecase import title
 
 from quodlibet.const import SUPPORT_EMAIL, COPYRIGHT
-from quodlibet.util.dprint import print_d, print_
-from .misc import environ, argv, cached_func, get_locale_encoding, \
-    get_fs_encoding
-from .environment import *
+from quodlibet.util.dprint import print_d, print_, print_e, print_w, print_exc
+from .misc import cached_func, get_module_dir, get_ca_file, \
+    get_locale_encoding, NamedTemporaryFile
+from .environment import is_plasma, is_unity, is_enlightenment, \
+    is_linux, is_openbsd, is_windows, is_wine, is_osx
 from .enum import enum
+from .i18n import _, C_, locale_format
 
 
 # pyflakes
-environ, argv, cached_func, get_locale_encoding, get_fs_encoding, enum
-
-
-if PY2:
-    def gdecode(s):
-        """Returns unicode for the glib text type"""
-
-        assert isinstance(s, bytes)
-        return s.decode("utf-8")
-else:
-    def gdecode(s):
-        """Returns unicode for the glib text type"""
-
-        assert isinstance(s, text_type)
-        return s
+cached_func, enum, print_w, print_exc, is_plasma, is_unity, is_enlightenment,
+is_linux, is_openbsd, is_windows, is_wine, is_osx, get_module_dir, get_ca_file,
+get_locale_encoding, NamedTemporaryFile
 
 
 class InstanceTracker(object):
@@ -107,10 +96,10 @@ class OptionParser(object):
 
     def __longs(self):
         longs = []
-        for long, arg in self.__args.items():
-            longs.append(long + (arg and "=" or ""))
-        for long, canon in self.__translate_long.items():
-            longs.append(long + (self.__args[canon] and "=" or ""))
+        for long_, arg in self.__args.items():
+            longs.append(long_ + (arg and "=" or ""))
+        for long_, canon in self.__translate_long.items():
+            longs.append(long_ + (self.__args[canon] and "=" or ""))
         return longs
 
     def __format_help(self, opt, space):
@@ -128,7 +117,7 @@ class OptionParser(object):
             l = max(l, len(k) + len(self.__args.get(k, "")) + 4)
 
         s = _("Usage: %(program)s %(usage)s") % {
-            "program": sys.argv[0],
+            "program": argv[0],
             "usage": self.__usage if self.__usage else _("[options]"),
         }
         s += "\n"
@@ -165,7 +154,7 @@ class OptionParser(object):
 
     def parse(self, args=None):
         if args is None:
-            args = sys.argv[1:]
+            args = argv[1:]
         from getopt import getopt, GetoptError
         try:
             opts, args = getopt(args, self.__shorts(), self.__longs())
@@ -182,7 +171,7 @@ class OptionParser(object):
                 text.append(
                     _("%r is not a unique prefix.") % s.split()[1])
             if "help" in self.__args:
-                text.append(_("Try %s --help.") % sys.argv[0])
+                text.append(_("Try %s --help.") % argv[0])
 
             print_e("\n".join(text))
             raise SystemExit(True)
@@ -232,17 +221,21 @@ def italic(string):
     return "<i>%s</i>" % string
 
 
-def parse_time(timestr, err=(ValueError, re.error)):
+def parse_time(timestr, err=object()):
     """Parse a time string in hh:mm:ss, mm:ss, or ss format."""
+
     if timestr[0:1] == "-":
         m = -1
         timestr = timestr[1:]
     else:
         m = 1
+
     try:
         return m * reduce(lambda s, a: s * 60 + int(a),
                           re.split(r":|\.", timestr), 0)
-    except err:
+    except (ValueError, re.error):
+        if err is None:
+            raise
         return 0
 
 
@@ -318,7 +311,24 @@ def parse_date(datestr):
     except IndexError:
         raise ValueError
 
-    return time.mktime(time.strptime(datestr, frmt))
+    try:
+        return time.mktime(time.strptime(datestr, frmt))
+    except OverflowError as e:
+        raise ValueError(e)
+
+
+def format_int_locale(value):
+    """Turn an integer into a grouped, locale-dependent string
+    e.g. 12345 -> "12,345" or "12.345" etc
+    """
+    return locale_format("%d", value, grouping=True)
+
+
+def format_float_locale(value, format="%.2f"):
+    """Turn a float into a grouped, locale-dependent string
+    e.g. 12345.67 -> "12,345.67" or "12.345,67" etc
+    """
+    return locale_format(format, value, grouping=True)
 
 
 def format_rating(value, blank=True):
@@ -339,22 +349,28 @@ def format_bitrate(value):
 
 
 def format_size(size):
-    """Turn an integer size value into something human-readable."""
+    """Turn an integer size value into something human-readable.
+
+    Args:
+        size (int): size in bytes
+    Returns:
+        text_type
+    """
     # TODO: Better i18n of this (eg use O/KO/MO/GO in French)
     if size >= 1024 ** 3:
-        return "%.1f GB" % (float(size) / (1024 ** 3))
+        return u"%.1f GB" % (float(size) / (1024 ** 3))
     elif size >= 1024 ** 2 * 100:
-        return "%.0f MB" % (float(size) / (1024 ** 2))
+        return u"%.0f MB" % (float(size) / (1024 ** 2))
     elif size >= 1024 ** 2 * 10:
-        return "%.1f MB" % (float(size) / (1024 ** 2))
+        return u"%.1f MB" % (float(size) / (1024 ** 2))
     elif size >= 1024 ** 2:
-        return "%.2f MB" % (float(size) / (1024 ** 2))
+        return u"%.2f MB" % (float(size) / (1024 ** 2))
     elif size >= 1024 * 10:
-        return "%d KB" % int(size / 1024)
+        return u"%d KB" % int(size / 1024)
     elif size >= 1024:
-        return "%.2f KB" % (float(size) / 1024)
+        return u"%.2f KB" % (float(size) / 1024)
     else:
-        return "%d B" % size
+        return u"%d B" % size
 
 
 def format_time(time):
@@ -380,6 +396,13 @@ def format_time_display(time):
     return format_time(time).replace(":", u"\u2236")
 
 
+def format_time_seconds(time):
+    from quodlibet import ngettext
+
+    time_str = format_int_locale(time)
+    return ngettext("%s second", "%s seconds", time) % time_str
+
+
 def format_time_long(time, limit=2):
     """Turn a time value in seconds into x hours, x minutes, etc.
 
@@ -387,6 +410,8 @@ def format_time_long(time, limit=2):
     0 means no limit.
     """
 
+    from quodlibet import ngettext
+
     if time < 1:
         return _("No time information")
 
@@ -416,6 +441,21 @@ def format_time_long(time, limit=2):
     return ", ".join(time_str)
 
 
+def format_time_preferred(t, fmt=None):
+    """Returns duration formatted to user's preference"""
+    from quodlibet.config import DurationFormat, DURATION
+    fmt = fmt or DURATION.format
+
+    if fmt == DurationFormat.STANDARD:
+        return format_time_long(t, 2)
+    elif fmt == DurationFormat.NUMERIC:
+        return format_time_display(t)
+    elif fmt == DurationFormat.FULL:
+        return format_time_long(t, 5)
+    else:
+        return format_time_seconds(t)
+
+
 def capitalize(str):
     """Capitalize a string, not affecting any character after the first."""
     return str[:1].upper() + str[1:]
@@ -450,44 +490,12 @@ def human_sort_key(s, normalize=unicodedata.normalize):
 def website(site):
     """Open the given URL in the user's default browser"""
 
-    if os.name == "nt" or sys.platform == "darwin":
-        return webbrowser.open(site)
+    from gi.repository import Gtk, Gdk, GLib
 
-    # all commands here return immediately
-    for prog in ["xdg-open", "gnome-open"]:
-        if not iscommand(prog):
-            continue
-
-        status = subprocess.check_call([prog, site])
-        if status == 0:
-            return True
-
-    # sensible-browser is a debian thing
-    blocking_progs = ["sensible-browser"]
-    blocking_progs.extend(environ.get("BROWSER", "").split(":"))
-
-    for prog in blocking_progs:
-        if not iscommand(prog):
-            continue
-
-        # replace %s with the url
-        args = prog.split()
-        for i, arg in enumerate(args):
-            if arg == "%s":
-                args[i] = site
-                break
-        else:
-            args.append(site)
-
-        # calling e.g. firefox blocks, so call async and hope for the best
-        try:
-            spawn(args)
-        except RuntimeError:
-            continue
-        else:
-            return True
-
-    return False
+    try:
+        Gtk.show_uri(None, site, Gdk.CURRENT_TIME)
+    except GLib.Error:
+        print_exc()
 
 
 def tag(name, cap=True):
@@ -503,7 +511,7 @@ def tag(name, cap=True):
             # Translators: If tag names, when capitalized, should not
             # be title-cased ("Looks Like This"), but rather only have
             # the first letter capitalized, translate this string as
-            # something other than "titlecase?".
+            # something non-empty other than "titlecase?".
             if C_("check", "titlecase?") == "titlecase?":
                 parts = map(title, parts)
             else:
@@ -536,6 +544,7 @@ def pattern(pat, cap=True, esc=False, markup=False):
     """
 
     from quodlibet.pattern import Pattern, XMLFromPattern, XMLFromMarkupPattern
+    from quodlibet.formats import FILESYSTEM_TAGS
 
     class Fakesong(dict):
         cap = False
@@ -548,6 +557,8 @@ def pattern(pat, cap=True, esc=False, markup=False):
         list_separate = list
 
         def __call__(self, tag, *args):
+            if tag in FILESYSTEM_TAGS:
+                return fsnative(text_type(tag))
             return 0 if '~#' in tag[:2] else self.comma(tag)
 
     fakesong = Fakesong({'filename': tag('filename', cap)})
@@ -576,10 +587,7 @@ def spawn(argv, stdout=False):
 
     from gi.repository import GLib
 
-    types = map(type, argv)
-    if not (min(types) == max(types) == str):
-        raise TypeError("executables and arguments must be str objects")
-    print_d("Running %r" % " ".join(argv))
+    print_d("Running %r" % argv)
     args = GLib.spawn_async(argv=argv, flags=GLib.SpawnFlags.SEARCH_PATH,
                             standard_output=stdout)
 
@@ -593,24 +601,10 @@ def fver(tup):
     return ".".join(map(str, tup))
 
 
-def uri_is_valid(uri):
-    return bool(urlparse(uri)[0])
-
-
 def make_case_insensitive(filename):
     return "".join(["[%s%s]" % (c.lower(), c.upper()) for c in filename])
 
 
-def print_exc(limit=None, file=None):
-    """A wrapper preventing crashes on broken pipes in print_exc."""
-    if file is None:
-        if PY2:
-            file = sys.stderr
-        else:
-            file = sys.stderr.buffer
-    print_(traceback.format_exc(limit=limit), output=file)
-
-
 class DeferredSignal(object):
     """A wrapper for connecting functions to signals.
 
@@ -669,6 +663,13 @@ class DeferredSignal(object):
     def __closure__(self):
         return self.func.__closure__
 
+    def call(self, *args):
+        """Force a call"""
+
+        self.abort()
+        self.args = args
+        self._wrap()
+
     def abort(self):
         """Abort any queued up calls.
 
@@ -758,7 +759,7 @@ class cached_property(object):
         self.fget = fget
         self.__doc__ = doc or fget.__doc__
         self.__name__ = name = fget.__name__
-        # these get name mangled, so caching wont work unless
+        # these get name mangled, so caching won't work unless
         # we mangle too
         assert not (name.startswith("__") and not name.endswith("__")), \
             "can't cache a dunder method"
@@ -781,7 +782,7 @@ def sanitize_tags(tags, stream=False):
         key = key.lower()
         key = {"location": "website"}.get(key, key)
 
-        if isinstance(value, unicode):
+        if isinstance(value, text_type):
             lower = value.lower().strip()
 
             if key == "channel-mode":
@@ -835,7 +836,7 @@ def sanitize_tags(tags, stream=False):
         if not stream and key in ("title", "album", "artist", "date"):
             continue
 
-        if isinstance(value, (int, long, float)):
+        if isinstance(value, number_types):
             if not key.startswith("~#"):
                 key = "~#" + key
             san[key] = value
@@ -843,7 +844,7 @@ def sanitize_tags(tags, stream=False):
             if key.startswith("~#"):
                 key = key[2:]
 
-            if not isinstance(value, unicode):
+            if not isinstance(value, text_type):
                 continue
 
             value = value.strip()
@@ -892,15 +893,10 @@ def limit_songs(songs, max, weight_by_ratings=False):
         return songs
     else:
         if weight_by_ratings:
-            def choose(r1, r2):
-                if r1 or r2:
-                    return cmp(random.random(), r1 / (r1 + r2))
-                else:
-                    return random.randint(-1, 1)
-
-            def rating(song):
-                return song("~#rating")
-            songs.sort(cmp=choose, key=rating)
+            def rating_weighted_random(song):
+                # Apply even (random : higher rating) weighting
+                return (1 - song("~#rating")) * (1 + random.random())
+            songs.sort(key=rating_weighted_random)
         else:
             random.shuffle(songs)
         return songs[:max]
@@ -947,18 +943,6 @@ def load_library(names, shared=True):
     else:
         load_func = ctypes.cdll.LoadLibrary
 
-    if is_osx():
-        # make sure it's either empty or contains /usr/lib.
-        # (jhbuild sets it for example). Otherwise ctypes can't
-        # find libc (bug?)
-        if "DYLD_FALLBACK_LIBRARY_PATH" in os.environ:
-            paths = os.environ["DYLD_FALLBACK_LIBRARY_PATH"]
-            paths = paths.split(os.pathsep)
-            if "/usr/lib" not in paths:
-                paths.append("/usr/lib")
-                os.environ["DYLD_FALLBACK_LIBRARY_PATH"] = \
-                    os.pathsep.join(paths)
-
     errors = []
     for name in names:
         dlopen_name = name
@@ -966,6 +950,9 @@ def load_library(names, shared=True):
                 ".dylib" not in name:
             dlopen_name = ctypes.util.find_library(name) or name
 
+        if is_osx() and not os.path.isabs(dlopen_name):
+            dlopen_name = os.path.join(sys.prefix, "lib", dlopen_name)
+
         try:
             return load_func(dlopen_name), name
         except OSError as e:
@@ -1118,11 +1105,22 @@ def set_process_title(title):
 
     try:
         libc = load_library(["libc.so.6", "c"])[0]
-        # 15 = PR_SET_NAME, apparently
-        libc.prctl(15, title, 0, 0, 0)
+        prctl = libc.prctl
     except (OSError, AttributeError):
         print_d("Couldn't find module libc.so.6 (ctypes). "
                 "Not setting process title.")
+    else:
+        prctl.argtypes = [
+            ctypes.c_int, ctypes.c_ulong, ctypes.c_ulong,
+            ctypes.c_ulong, ctypes.c_ulong,
+        ]
+        prctl.restype = ctypes.c_int
+
+        PR_SET_NAME = 15
+        data = ctypes.create_string_buffer(title.encode("utf-8"))
+        res = prctl(PR_SET_NAME, ctypes.addressof(data), 0, 0, 0)
+        if res != 0:
+            print_w("Setting the process title failed")
 
 
 def list_unique(sequence):
